function SXFINDPAR, hdr, name, COUNT=matches, COMMENT = comments, $
  NoContinue = NoContinue, SILENT = silent
  ;Verifies the existence of a keyword in a header 
  ;Adapted from astrolib sxpar, Jonathan Gagne, April 25 2014.

  if N_params() LT 2 then begin
    print,'Syntax -     result =  sxpar( hdr, name, [abort])'
    print,'   Input Keywords:    /NOCONTINUE, /SILENT'
    print,'   Output Keywords:   COUNT=,  COMMENT= '
    return, -1
  endif
  
  VALUE = 0
  abort = 'FITS Header'
  abort_return = 1
  if abort_return then On_error,1 else On_error,2

  ;       Check for valid header

  s = size(hdr)         ;Check header for proper attributes.
  if ( s[0] NE 1 ) or ( s[2] NE 7 ) then $
    message,'FITS Header (first parameter) must be a string array'

  nam = strtrim( strupcase(name) )      ;Copy name, make upper case


  ;  Determine if NAME is of form 'keyword*'.  If so, then strip off the '*', and
  ;  set the VECTOR flag.  One must consider the possibility that NAM is an empty
  ;  string.

  namelength1 = (strlen(nam) - 1 ) > 1
  if strpos( nam, '*' ) EQ namelength1 then begin
    nam = strmid( nam, 0, namelength1)
    vector = 1                  ;Flag for vector output
    name_length = strlen(nam)   ;Length of name
    num_length = 8 - name_length        ;Max length of number portion
    if num_length LE 0 then  $
      message, 'Keyword length must be 8 characters or less'

    ;  Otherwise, extend NAME with blanks to eight characters.

  endif else begin
    while strlen(nam) LT 8 do nam = nam + ' ' ;Make 8 chars long
    vector = 0
  endelse


  ;  If of the form 'keyword*', then find all instances of 'keyword' followed by
  ;  a number.  Store the positions of the located keywords in NFOUND, and the
  ;  value of the number field in NUMBER.

  histnam = (nam eq 'HISTORY ') or (nam eq 'COMMENT ') or (nam eq '')
  if N_elements(start) EQ 0 then start = -1l
  start = long(start[0])
  if (not vector) and (start GE 0) then begin
    if N_elements(precheck)  EQ 0 then precheck = 5
    if N_elements(postcheck) EQ 0 then postcheck = 20
    nheader = N_elements(hdr)
    mn = (start - precheck)  > 0
    mx = (start + postcheck) < nheader-1
    keyword = strmid(hdr[mn:mx], 0, 8)
  endif else begin
    restart:
    start   = -1l
    keyword = strmid( hdr, 0, 8)
  endelse

  if vector then begin
    nfound = where(strpos(keyword,nam) GE 0, matches)
    if ( matches gt 0 ) then begin
      numst= strmid( hdr[nfound], name_length, num_length)
      number = replicate(-1, matches)
      for i = 0, matches-1 do         $
        if VALID_NUM( numst[i], num,/INTEGER) then number[i] = num
      igood = where(number GE 0, matches)
      if matches GT 0 then begin
        nfound = nfound[igood]
        number = number[igood]
      endif
    endif

    ;  Otherwise, find all the instances of the requested keyword.  If more than
    ;  one is found, and NAME is not one of the special cases, then print an error
    ;  message.

  endif else begin
    nfound = where(keyword EQ nam, matches)
    if (matches EQ 0) and (start GE 0) then goto, RESTART
    if (start GE 0) then nfound = nfound + mn
    if (matches GT 1) and (not histnam) then        $
      if not keyword_set(silent) then $
      message,/informational, 'Warning - keyword ' +   $
      nam + ' located more than once in ' + abort
    if (matches GT 0) then start = nfound[matches-1]
  endelse


  ; Process string parameter

  if matches GT 0 then begin
    line = hdr[nfound]
    svalue = strtrim( strmid(line,9,71),2)
    if histnam then $
      value = strtrim(strmid(line,8,71),2) else for i = 0,matches-1 do begin
      if ( strmid(svalue[i],0,1) EQ "'" ) then begin   ;Is it a string?
        test = strmid( svalue[i],1,strlen( svalue[i] )-1)
        next_char = 0
        off = 0
        value = ''
        NEXT_APOST:
        endap = strpos(test, "'", next_char)      ;Ending apostrophe
        if endap LT 0 then $
          MESSAGE,'Value of '+name+' invalid in '+abort
        value = value + strmid( test, next_char, endap-next_char )

        ;  Test to see if the next character is also an apostrophe.  If so, then the
        ;  string isn't completed yet.  Apostrophes in the text string are signalled as
        ;  two apostrophes in a row.

        if strmid( test, endap+1, 1) EQ "'" then begin
          value = value + "'"
          next_char = endap+2
          goto, NEXT_APOST
        endif

        ; Extract the comment, if any

        slash = strpos( test, "/", endap )
        if slash LT 0 then comment = '' else    $
          comment = strmid( test, slash+1, strlen(test)-slash-1 )

        ; This is a string that could be continued on the next line.  Check this
        ; possibility with the following four criteria: *1) Ends with '&'
        ; (2) Next line is CONTINUE  (3) LONGSTRN keyword is present (recursive call to
        ; SXPAR) 4. /NOCONTINE is not set

        if not keyword_set(nocontinue) then begin
          off = off + 1
          val = strtrim(value,2)

          if (strlen(val) gt 0) and $
            (strmid(val, strlen(val)-1, 1) EQ '&') and $
            (strmid(hdr[nfound[i]+off],0,8) EQ 'CONTINUE') then begin
            if (size(sxpar(hdr, 'LONGSTRN',/NoCONTINUE)))[1] EQ 7 then begin
              value = strmid(val, 0, strlen(val)-1)
              test = hdr[nfound[i]+off]
              test = strmid(test, 8, strlen(test)-8)
              test = strtrim(test, 2)
              if strmid(test, 0, 1) NE "'" then message, $
                'ERROR: Invalidly CONTINUEd string in '+ abort
              next_char = 1
              GOTO, NEXT_APOST
            ENDIF
          ENDIF
        ENDIF


        ; Process non-string value

      endif else begin

        test = svalue[i]
        slash = strpos( test, "/" )
        if slash GT 0 then begin
          comment = strmid( test, slash+1, strlen(test)-slash-1 )
          test = strmid( test, 0, slash )
        end else comment = ''

        ; Find the first word in TEST.  Is it a logical value ('T' or 'F')

        test2 = test
        value = gettok(test2,' ')
        if ( value EQ 'T' ) then value = 1b else $
          if ( value EQ 'F' ) then value = 0b else begin

          ;  Test to see if a complex number.  It's  a complex number if the value and
          ;  the next word, if any, are both valid values.

          if strlen(test2) EQ 0 then goto, NOT_COMPLEX
          value2 = gettok( test2, ' ')
          if value2 EQ '' then goto, NOT_COMPLEX
          On_ioerror, NOT_COMPLEX
          value2 = float(value2)
          value = complex(value,value2)
          goto, GOT_VALUE

          ;  Not a complex number.  Decide if it is a floating point, double precision,
          ;  or integer number.

          NOT_COMPLEX:
          On_IOerror, GOT_VALUE
          if (strpos(value,'.') GE 0) or (strpos(value,'E') GT 0) $
            or (strpos(value,'D') GE 0) then begin  ;Floating or double?
            if ( strpos(value,'D') GT 0 ) or $  ;Double?
              ( strlen(value) GE 8 ) then value = double(value) $
            else value = float(value)
          endif else begin                   ;Long integer
            lmax = 2.0d^31 - 1.0d
            lmin = -2.0d31
            value = double(value)
            if (value GE lmin) and (value LE lmax) then $
              value = long(value)
          endelse

          GOT_VALUE:
          On_IOerror, NULL
        endelse
      endelse; if c eq apost

      ;  Add to vector if required

      if vector then begin
        if ( i EQ 0 ) then begin
          maxnum = max(number)
          dtype = size(value,/type)
          result = make_array( maxnum, TYPE = dtype )
          comments = strarr( maxnum )
        endif
        if size(value,/type) GT dtype then begin   ;Do we need to recast?
          result = result + 0*value
          dtype = size(value,/type)
        endif
        result[ number[i]-1 ] =  value
        comments[ number[i]-1 ] = comment
      endif else $
        comments = comment
    endfor

    if vector then begin
      !ERR = matches
      return, result
    endif else !ERR = 0

  endif  else  begin
    if abort_return then return, 0
    !ERR = -1
  endelse

  return, 1

END
