Pro astrometrynet_API, fits_files, TRUST_WCS=trust_wcs, FSTART=fstart, CONFIG_PATH=config_path, SERVER=server, $
  RA0=ra0, DEC0=dec0, RADIUS=radius, PLATESCALE=platescale, E_PLATESCALE=e_platescale, PARITY=parity, DEGREE=degree, $
  EXTRACMD=extracmd, TIMEOUT=timeout, API_KEY=api_key, PYTHON_PATH=python_path, CLIENT_PATH=client_path, JSON_PATH=json_path, $
  TMPDIR_PATH=tmpdir_path, VERIFY_WCS=verify_wcs, PRIVATE=private, RECURSIVE=recursive, SKIP=skip, DONOTTRUST=donottrust, $
  EXTEN_NO=exten_no, OVERWRITE=overwrite

;+
; NAME:
;       ASTROMETRYNET_API
;
; PURPOSE:
;       Update a fits file's WCS header using the "astrometrynet" API function.
;       This service uses tools provided by Astrometry.net. For more details, visit http://astrometry.net/
;
; EXPLANATION:
;       The fits file will be uploaded on supernova.astrometry.net, then solved for a valid astrometry. The resulting WCS file
;         will then be downloaded upon success, and used to update the original fits file's header. A keyword ASTRNET will be set
;         to 1 in the fits file's header. If the astrometry fails, a keyword ASTRNET will be set to 0 in the fits file's header. 
;       
;       More information on the astrometrynet project can be found here : http://astrometry.net/summary.html
;         as well as in this paper : http://adsabs.harvard.edu/abs/2010AJ....139.1782L
;       
;       More information on astrometrynet's API can be found here : http://supernova.astrometry.net/api_help
;       
;       A short explanation on the principles behind astrometry.net can be found here : http://dx.doi.org/10.6084/m9.figshare.841644
;       
;       More information on the Python API client can be found in the header of the "client.py" file (see REQUIREMENTS #5)
;
; REQUIREMENTS:
;       (1) A valid API key must be obtained by creating an account on supernova.astrometry.net and visiting the page http://supernova.astrometry.net/api_help
;             Note that your API key can change from time to time.
;       (2) A config.dat file must be passed to this program in order for it to be functional. It can either be passed through the CONFIG_PATH
;             header keyword or the $ASTROMETRYNET_CONFIG system variable. The configuration file must be a text file, and contain the following informations :
;             
;          API_KEY = rlybxguijlwrtepq ;THIS IS YOUR PERSONAL API KEY (SEE REQUIREMENTS #1 IN THE HEADER OF ASTROMETRYNET_API.PRO)
;          PYTHON_PATH = python ;THIS IS THE PATH TO PYTHON ON YOUR MACHINE
;          CLIENT_PATH = /Users/gagne/Documents/Python/Python_Library/03 - Astrometrynet/ ;THIS IS THE PATH TO THE PYTHON API CLIENT ROUTINE ON YOUR MACHINE
;          JSON_PATH = /Users/gagne/Documents/Python/Python_Library/01 - Library/simplejson-3.4.0/ ;THIS IS THE PATH TO THE SIMPLEJSON PYTHON LIBRARY ON YOUR MACHINE
;          TMPDIR_PATH = tmp/ ;THIS IS THE PATH TO A DIRECTORY ON YOUR MACHINE WHICH WILL BE USED AS A TEMPORARY REPOSITORY
;          SERVER = http://supernova.astrometry.net/api/ ;THIS CONTAINS THE SERVER
;          PRIVATE = 0 ;WHETHER YOUR IMAGE WILL BE PUBLICLY VISIBLE ON THE ASTROMETRYNET WEBSITE
;          PARITY = 1 ;DEFAULT VALUE FOR THE PARITY KEYWORD
;        
;       (3) Python must be installed on the system. See https://www.python.org/download/
;       (4) The "simplejson" Python library must be installed. See https://pypi.python.org/pypi/simplejson/
;       (5) The "client.py" Python procedure provided by Astrometrynet must be installed. See http://trac.astrometry.net/browser/trunk/src/astrometry/net/client/
;       (6) You must cite Lang et al. (2010; http://adsabs.harvard.edu/abs/2010AJ....139.1782L) when you use this code.
;       (7) You must include the following reference when you use this code : "This service uses tools provided by Astrometry.net. For more details, visit http://astrometry.net/"
;
; CALLING SEQUENCE:
;       ASTROMETRYNET_API, fits_files, [ FSTART=fstart, CONFIG_PATH=config_path, SERVER=server, $
;                          RA0=ra0, DEC0=dec0, RADIUS=radius, PLATESCALE=platescale, E_PLATESCALE=e_platescale, PARITY=parity, DEGREE=degree, $
;                          EXTRACMD=extracmd, TIMEOUT=timeout, API_KEY=api_key, PYTHON_PATH=python_path, CLIENT_PATH=client_path, JSON_PATH=json_path, $
;                          TMPDIR_PATH=tmpdir_path, VERIFY_WCS=verify_wcs, /PRIVATE, /RECURSIVE, /TRUST_WCS, /SKIP ]
;
; INPUTS:
;       FITS_FILES = The path to a fits file, or a directory containing multiple fits files. In the latter case, the program will run on all fits files. 
;
; OPTIONAL INPUT:
;       FSTART = In the case where FITS_FILE is set as a fits file directory, this is the file number from which to start the astrometry.
;                Preceding files will be ignored.  
;       CONFIG_PATH = This contains the path to the configuration file (see REQUIREMENTS #2)
;       SERVER = The server which is to be used to build the astrometric solution, usually
;         http://nova.astrometry.net/api/ or http://supernova.astrometry.net/api/ (Default value is SERVER key in the CONFIG file - See REQUIREMENTS #2) 
;       RA0 = Estimated position of the submitted field in right ascention (degrees).
;             This is passed to client.py as the --ra keyword.
;       DEC0 = Estimated position of the submitted field in declination (degrees).
;              This is passed to client.py as the --dec keyword.
;       RADIUS = Radius (in degrees) to search for around the estimated field position.
;                This is passed to client.py as the --radius keyword.
;       PLATESCALE = Estimated platescale for the input image, in arcseconds per pixel.
;                    This keyword (and E_PLATESCALE) are passed to client.py as the [X] keywords, adding "--scale-units arcsecperpix".
;       E_PLATESCALE = Error on the estimated platescale.
;                      This keyword (and PLATESCALE) are passed to client.py as the [X] keywords, adding "--scale-units arcsecperpix".
;       PARITY = Scalar string specifying which image parity to try. (1 for positive or 0 for negative, default is trying both).
;                This is passed to client.py as the --parity keyword. Generally, fits files will have a positive parity (1)
;                and photographies will have a negative parity (0)
;       DEGREE = Set the polynomial degree for the astrometry solution. The default is 2. This is passed to client.py as [X]
;       EXTRACMD = Scalar string containing extra commands to pass to the Python code client.py. Use correct syntax (refer to 
;                  the contents of client.py).
;       TIMEOUT = Total time (in minutes) before the astrometrynet child process is killed (if no solution has been found).
;                 Default is no time limit.
;       API_KEY = Your personal API key for the astrometrynet service. This keyword takes precedence over the value found in the configuration file.
;       PYTHON_PATH = The path to Python on your machine. This keyword takes precedence over the value found in the configuration file.
;       CLIENT_PATH = The path to the Python API client on your machine. This keyword takes precedence over the value found in the configuration file.
;       JSON_PATH = The path to the simplejson library on your machine. This keyword takes precedence over the value found in the configuration file.
;       TMPDIR_PATH = A temporary directory to store files on your machine. This keyword takes precedence over the value found in the configuration file.
;       VERIFY_WCS = Verify fits file astrometry prior to making a new one.
;
; OPTIONAL INPUT KEYWORD:
;       /TRUST_WCS - If this keyword is set, the astrometry already present in the fits file will be used to fasten the procedure.
;                    The plate scale will be trusted to ± 5% and the field position to ± 12 arcminutes. 
;       /PRIVATE = Whether or not your image will be visible on the astrometrynet website. 
;                  This keyword takes precedence over the value found in the configuration file.
;       /RECURSIVE = Make astrometry on all fits files in subdirectories (only relevant when FITS_FILES keyword is a directory itself).
;       /SKIP = Skip files with a good astrometry from astrometrynet (ASTRNET = 1 in their headers)
;       
; NOTE:
;
; This routine returns SIP polynomials describing the image distortion, understood by the astrolib and documented on
; http://tdc-www.harvard.edu/wcstools/SIP_distortion_v1_0.pdf
;
; PROCEDURES USED:
;       READCOL, READFITS(), SXPAR(), XY2AD, MKHDR, WRITEFITS, SPAWN_ASTROMETRYNET_API, HEADFITS(), SXADDPAR, FILE_EXT(),
;       STRKILL, EXTAST, PUTAST, TRANSFER_ALL_HDR_KEYS, SXADDHIST, STRMID_ARR(), STRPOS_ARR()
;
; MODIFICATION HISTORY:
;       WRITTEN, Jonathan Gagne, April, 25 2014
;       MODIFIED, Jonathan Gagne, April 30, 2014 : Changed structure of spawned command for a more efficient catch of the Python child process PID
;-
  
  forward_function readcol, readfits, sxpar, xy2ad, mkhdr, writefits, spawn_astrometrynet_api, headfits, sxaddpar, file_ext, strkill $
                 , extast, putast, transfer_all_hdr_keys, sxaddhist, strmid_arr, strpos_arr
  
  ;Is the configuration file needed ?
  if ~(keyword_set(api_key) and keyword_set(python_path) and keyword_set(client_path) and keyword_set(json_path) and $
    keyword_set(tmpdir_path)) then begin
    
    ;Find the configuration file
    if ~keyword_set(config_path) then begin
      spawn, 'echo $ASTROMETRYNET_CONFIG', out
      if strtrim(out[0L],2L) eq '' then message, ' You must input a config file path ! See REQUIREMENT #2 in the header of astrometrynet_api.pro !'
      config_path = strtrim(out[0L],2L)
    endif
    
    ;Read configuration file
    if ~file_test(config_path) then $
      message, ' The configuration file '+config_path+' could not be found !'
    
    readcol, config_path, format='A,A', config_name, config_value, comment=';', delimiter='=', /silent
    bad = where(strpos(config_value,';') ne -1L, nbad)
    if nbad ne 0L then $
      config_value[bad] = strmid_arr(config_value[bad], 0L, strpos_arr(config_value[bad],';'))
    config_name = strtrim(config_name,2L)
    config_value = strtrim(config_value,2L)
    
  endif
  
  ;Interpret needed configuration file entries
  if ~keyword_set(api_key) then begin
    ind = (where(strupcase(strtrim(config_name,2L)) eq 'API_KEY', nf))[0L]
    if nf eq 0L then message, ' The API_KEY variable was not found in the configuration file !'
    api_key = config_value[ind]
  endif
  if ~keyword_set(python_path) then begin
    ind = (where(strupcase(strtrim(config_name,2L)) eq 'PYTHON_PATH', nf))[0L]
    if nf eq 0L then message, ' The PYTHON_PATH variable was not found in the configuration file !'
    python_path = config_value[ind]
  endif
  if ~keyword_set(client_path) then begin
    ind = (where(strupcase(strtrim(config_name,2L)) eq 'CLIENT_PATH', nf))[0L]
    if nf eq 0L then message, ' The CLIENT_PATH variable was not found in the configuration file !'
    client_path = config_value[ind]
  endif
  if ~keyword_set(tmpdir_path) then begin
    ind = (where(strupcase(strtrim(config_name,2L)) eq 'TMPDIR_PATH', nf))[0L]
    if nf eq 0L then message, ' The TMPDIR_PATH variable was not found in the configuration file !'
    tmpdir_path = config_value[ind]
  endif
  if ~keyword_set(json_path) then begin
    ind = (where(strupcase(strtrim(config_name,2L)) eq 'JSON_PATH', nf))[0L]
    if nf eq 0L then message, ' The JSON_PATH variable was not found in the configuration file !'
    json_path = config_value[ind]
  endif
  if ~keyword_set(server) and keyword_set(config_name) then begin
    ind = (where(strupcase(strtrim(config_name,2L)) eq 'SERVER', nf))[0L]
    if nf eq 0L then message, ' The SERVER variable was not found in the configuration file ! Setting to http://supernova.astrometry.net', /continue else $
      server = config_value[ind]
  endif
  if ~keyword_set(server) then server = 'http://supernova.astrometry.net/api/'
  if strmid(server,strlen(server)-1L) ne '/' then server += '/'
  if ~keyword_set(private) and keyword_set(config_name) then begin
    ind = (where(strupcase(strtrim(config_name,2L)) eq 'PRIVATE', nf))[0L]
    if nf eq 0L then message, ' The PRIVATE variable was not found in the configuration file ! Setting to default (public)', /continue else $
      private = fix(config_value[ind])
  endif
  if ~keyword_set(private) then private = 0
  if ~keyword_set(parity) and keyword_set(config_name) then begin
    ind = (where(strupcase(strtrim(config_name,2L)) eq 'PARITY', nf))[0L]
    if nf ne 0L then $
      parity = fix(config_value[ind])
  endif
  
  ;Make sure that the temporary directory exists
  file_mkdir, tmpdir_path
  
  ;Define a temporary file for WCS download
  tmp_wcs = tmpdir_path+'wcs_tmp.wcs'
  
  ;Check if there are multiple fits files
  nf = n_elements(fits_files)
  
  ;Check if a directory was input
  if nf eq 1L and strmid(fits_files[0],strlen(fits_files[0])-1L) eq path_sep() then begin
    print, ' Detected a directory ! Using all fits file inside it !'
    if keyword_set(recursive) then $
      ff = file_search(fits_files[0],'*.fits*') else $
      ff = file_search(fits_files[0]+'*.fits*')
    if ff[0] eq '' then message, ' No fits files found !'
    fits_files = ff
    nf = n_elements(fits_files)
  endif
  
  ;Interpret FSTART keyword
  if keyword_set(fstart) and nf gt 1L then j0 = fstart else j0 = 0L
  
  ;Loop on all FITS FILES
  for j=j0, nf-1L do begin
    
    ;Define a temporary file for image upload
    tmp_upload = tmpdir_path+'tmp_out'+strjoin(strjoin(strsplit(systime( /utc ),' :',/ext),'-'))+'_'+strtrim(round(frac(systime(/seconds))*1d6),2L)+'.fits'
    
    fits_file = fits_files[j]
    txtheader = ' API Astrometry for file ['+strtrim(j+1L,2L)+'/'+strtrim(nf,2L)+'] : '+file_basename(fits_file)+' ...'
    
    ;Read fits image 
    im = readfits(fits_file, hdr0, /silent,EXTEN_NO=exten_no)
    if n_elements(im) eq 1L then $
      message, ' Could not read FITS file ! Are you trying to read the wrong extension ?'
      
    ;Skip files with good astrometry from astrometry.net if /SKIP is specified
    if keyword_set(skip) then $
      if sxpar(hdr0,'ASTRNET') eq 1 then continue
    
    ;Extract astrometry if keyword /TRUST_WCS is used
    if keyword_set(trust_wcs) then begin
      extast, hdr0, astr0
      if size(astr0,/type) eq 8L then begin
        platescale = sqrt(abs(determ(astr0.cd)))*3600.
        e_platescale = 0.5;%
        midx = float(sxpar(hdr0,'NAXIS1'))/2.
        midy = float(sxpar(hdr0,'NAXIS2'))/2.
        xy2ad, midx, midy, astr0, ra0, dec0
        radius = 0.2
        verify_wcs = 1
      endif else begin
        
      endelse
    endif
    
    ;Create a temporary header to avoid BITPIX problems on astrometrynet
    mkhdr, hdr0t, im
    
    ;Input the astrometry if WCS is to be verified
    if keyword_set(verify_wcs) then $
      putast, hdr0t, astr0
    
    ;Write the temporary fits file
    writefits, tmp_upload, im, hdr0t, /compress
    
    ;Add simplejson to the Python path if it's not already there
    if strmid(json_path,strlen(json_path)-1L) eq path_sep() then $
      json_path = strmid(json_path,0L,strlen(json_path)-1L)
    spawn, 'echo $PYTHONPATH', def_python
    if strpos(def_python,json_path) eq -1L then $
      def_python = json_path + ':' +def_python[0L]
    cmd = 'export PYTHONPATH="'+def_python[0L]+'" > /dev/null; '
    
    ;Build Python command
    cmd += python_path+' "'+client_path+'client.py" --server "'+server+'" --apikey '+api_key
    if keyword_set(private) then $
      cmd += ' -p'
    cmd += ' --upload "'+tmp_upload+'.gz"'
    if keyword_set(parity) then $
      cmd += ' --parity '+strtrim(parity,2L)
    cmd += ' --wcs "'+tmp_wcs+'"'
    if keyword_set(ra0) then $
      cmd += ' --ra '+strtrim(ra0,2) 
    if keyword_set(dec0) then $
      cmd += ' --dec '+strtrim(dec0,2)
    if keyword_set(radius) then $
      cmd += ' --radius '+strtrim(radius,2)
    if keyword_set(platescale) then begin
      if ~keyword_set(e_platescale) then e_platescale = 5.
      cmd += ' --scale-units arcsecperpix --scale-est '+strtrim(platescale,2)+' --scale-err '+strtrim(e_platescale,2)
    endif 
     
    print, txtheader
    print, 'Uploading & building astrometry ...'
    
    ;Spawn the API client
    spawn_astrometrynet_api, cmd, STATUS=status, TXTHEADER=txtheader, TIMER=timeout
    
    ;Verify astrometry status
    if status ne -2 then begin
      print, ' Failed !'
      hh = headfits(fits_file)
      sxaddpar, hh, 'ASTRNET', 0
      if file_ext(fits_file) eq '.gz' then begin
        strkill, fits_file, '.gz'
        if keyword_set(overwrite) then output_file = fits_file else $
          output_file = file_dirname(fits_file)+path_sep()+file_basename(fits_file,file_ext(fits_file))+'_astrnet.fits'
        writefits, output_file, im, hh, /COMPRESS
      endif else begin
        if keyword_set(overwrite) then output_file = fits_file else $
          output_file = file_dirname(fits_file)+path_sep()+file_basename(fits_file,file_ext(fits_file))+'_astrnet.fits'
        writefits, output_file, im, hh
      endelse
      file_delete, tmp_upload+'.gz'
  	  continue
    endif
    
    ;Read output WCS file
    hdr = headfits(tmp_wcs)
    
    ;Extract astrometry and put in the fits file
    extast, hdr, astr
    hdr_out = hdr0
    putast, hdr_out, astr
    transfer_all_hdr_keys, hdr, hdr_out, /COMMENTS, /HISTORY
    sxaddhist, 'Processed with astrometrynet_API.pro on '+strjoin(strjoin(strsplit(systime( /utc ),' ',/ext),'-'))+' UTC', hdr_out
    sxaddpar, hdr_out, 'ASTRNET', 1
    
    ;Write output fits file
    if file_ext(fits_file) eq '.gz' then begin
      strkill, fits_file, '.gz'
      if keyword_set(overwrite) then output_file = fits_file else $
        output_file = file_dirname(fits_file)+path_sep()+file_basename(fits_file,file_ext(fits_file))+'_astrnet.fits'
      writefits, output_file, im, hdr_out, /COMPRESS
    endif else begin
      if keyword_set(overwrite) then output_file = fits_file else $
        output_file = file_dirname(fits_file)+path_sep()+file_basename(fits_file,file_ext(fits_file))+'_astrnet.fits'
      writefits, output_file, im, hdr_out
    endelse
    file_delete, tmp_upload+'.gz'
  endfor
  
  print, 'Done !'
End